angular.module('demoWidget4Module')

	/**
	 * Controller for the custom widget
	 * The controller provides the widget variables and manages the interactions between data-model and view
	 */
    .controller('demoWidget4Ctrl',['$scope','formatDateService','formatNumberService','thresholdConstants','thresholdService',
        function($scope, formatDateService, formatNumberService, thresholdConstants, thresholdService){

            //These services are providing value transformation functions that are used in the custom widget view
            $scope.formatDateService = formatDateService;
            $scope.formatNumberService = formatNumberService;

            var thresholds = [];

            /**
             * The init function creates all necessary scope variables at widget start up time
             */
            var initWidget = function() {
                //Create a scope object "config" if not present already
                //$scope.config holds the data of the widget settings like the assigned column(s)
                if(!$scope.config){
                    $scope.config = {};
                }
                //Define a property "assignedColumns" in the widget settings
                //to manage the information about the assigned column(s) in the controller
                if (!$scope.config.assignedColumns) {
                    $scope.config.assignedColumns = {};
                }

                //Define a property "advancedWidgetCfg" in the widget settings
                //to manage the information about the advanced column(s) configuration in the controller
                if (!$scope.config.advancedWidgetCfg) {
                    $scope.config.advancedWidgetCfg = {
                        thresholds: angular.copy(thresholdConstants.defaultThresholds)
                    };
                }
            };

            /**
             * This function searches for a configured column with a given data source column name
             * @param columnName The name of the data source column
             * @retrun The configured column, or undefined if not found
             */
            var getConfiguredColumnByName = function(columnName) {
                //check numeric column
                if($scope.config.assignedColumns.numDataColumn != undefined && $scope.config.assignedColumns.numDataColumn.newName == columnName){
                    return $scope.config.assignedColumns.numDataColumn;
                }
                //check text column
                if($scope.config.assignedColumns.textDataColumn != undefined && $scope.config.assignedColumns.textDataColumn.newName == columnName){
                    return $scope.config.assignedColumns.textDataColumn;
                } else
                //check date column
                if($scope.config.assignedColumns.dateDataColumn != undefined && $scope.config.assignedColumns.dateDataColumn.newName == columnName){
                    return $scope.config.assignedColumns.dateDataColumn;
                } else
                //check further columns
                if($scope.config.assignedColumns.furtherColumns != undefined){
                    for(var i = 0; i < $scope.config.assignedColumns.furtherColumns.length; i++){
                        var column = $scope.config.assignedColumns.furtherColumns[i];
                        if(column.newName == columnName){
                            return $scope.config.assignedColumns.furtherColumns[i];
                        }
                    }
                }
                return undefined;
            };

            /**
             * This function adds all configured columns to a scope variable configuredColumns
             * that can be used by the custom widget view to access the column configuration
             * @param data The data for the assigned and server-requested data source columns
             */
            var createConfiguredColumns = function(data) {
                for (var i = 0; i < data.columns.length; i++) {
                    var configuredColumn = getConfiguredColumnByName(data.columns[i].name);
                    $scope.configuredColumns[data.columns[i].name] = configuredColumn;
                }
            };

            /**
             * This function transforms the column und row data coming from the server
             * and creates scope variables that can be easily used in the view
             * @param data The data for the assigned and server-requested data source columns
             */
            var createScopeVariables = function(data) {
                $scope.columns = data.columns;
                $scope.rows = data.rows;
                $scope.rowCount = data.rows.length;
                $scope.configuredColumns = [];
            };

            /**
             * This function returns the threshold color for a matching value
             * @param thresholdIndex The row index pointing to the corresponding threshold in the threshold array
             * @param row The row data object
             * @return the threshold color or undefined if not found
             */
            $scope.getRowColor = function(thresholdIndex, row){
                if(thresholds[thresholdIndex]) {
                    var thresholdColumn = $scope.config.assignedColumns.numDataColumn;
                    if (thresholdColumn != undefined) {
                        var thresholdColumnIdx = -1;
                        for (var i = 0; i < $scope.columns.length; i++) {
                            if ($scope.columns[i].name == thresholdColumn.newName) {
                                thresholdColumnIdx = $scope.columns[i].idx;
                                break;
                            }
                        }
                        var thresholdColumnCellValue = row.values[thresholdColumnIdx];
                        for (var i = 0; i < thresholds[thresholdIndex].length; i++) {
                            if (thresholds[thresholdIndex][i].containsValue(thresholdColumnCellValue)) {
                                return thresholds[thresholdIndex][i].colour;
                            }
                        }
                    }
                }
                return undefined; //'transparent';
            }

            /**
             * This function collects the threshold colors for all rows
             * @param data The data for the assigned and server-requested data source columns (feed result)
             */
            var createThresholds = function(data) {
                thresholds = [];

                for(var i = 0; i < data.rows.length; i++){
                    var row = data.rows[i];
                    if($scope.config.advancedWidgetCfg.thresholds && data) {
                        var thresholdAreas = thresholdService.getThresholdAreas($scope.config.advancedWidgetCfg.thresholds, data, row);
                        thresholds.push(thresholdAreas);
                    }
                }
            };

            /**
             * This function is called from the custom widget framework when the requested data from the server returns
             * @param data The data for the assigned and server-requested data source columns (feed result)
             */
            $scope.setData = function(feedResult){
                createScopeVariables(feedResult);
                createConfiguredColumns(feedResult);
                createThresholds(feedResult);
            };

            /**
             * This function loads the custom widget settings at widget start up time and stores it in the controller
             * @param config The persisted custom widget settings from the dashboard definition
             */
            $scope.setConfig = function(config){
                $scope.config = config;
            };

            initWidget();
        }
    ]);