angular.module('demoWidget4Module')

    /**
     * Data service for the custom widget
     * The data service defines the data mapping - the structure of the data delivered by the server
     */
    .service('demoWidget4DataService',['widgetDataService','formatNumberService','typeconstants',
        function(widgetDataService,formatNumberService,typeconstants) {

            /**
             * Utility function createDataMappingColumn(...) creates one data mapping column
             * @param name The column name in the source, use dataColumn.name
             * @param type The data type of the column ("TEXT", "DATE", "NUMERIC")
             * @param filter Not interesting for this use case, set to '' or undefined
             * @param copyFrom If the user wants to have a source column twice, e.g. to see different aggregations for a numeric column, here he has to define the original column name
             * @param newName If the user has changed the column name please provide dataColumn.newName
             * @param round For numeric columns the user can define if values should be rounded or not. Set to true or false
             * @param precision Specifies precision for numeric columns. There is a utility function in formatNumberService that is called getPrecisionFromFormat
             * that can be used to calculate the precision from a format pattern. Set to undefined for text or date columns.
             * @param keepInResult Not interesting for this use case, set to true
             * @param aggregation Aggregation, if not specified the aggregation in the datamapping column is undefined
             * @returns A datamapping column object
             */
            var createDataMappingColumn =
                function(dataColumn){
                    var isNumericColumn =  dataColumn.type === typeconstants.NUMBER;
                    var format = isNumericColumn ? formatNumberService.getFormatFromFormatKey(dataColumn.format) : undefined,
                        precision = isNumericColumn ? formatNumberService.getPrecisionFromFormat(format) : undefined,
                        aggregation = isNumericColumn && dataColumn.aggregation ? {type: dataColumn.aggregation} : undefined;
                    return widgetDataService.
                        createDataMappingColumn(
                            dataColumn.name,
                            dataColumn.type,
                            null,
                            '',
                            dataColumn.newName || '',
                            dataColumn.round,
                            precision,
                            true,
                            aggregation);
                };

            return {

                /**
                  * This function is called by the custom widget framework.
                  * It prepares the data handling of the custom widget (item) with the server
                  * by creating the data mapping for each configured (assigned) data column
                  */
                getDataMapping : function(item) {
                    var dataMapping = {
                            columns: []
                        }
                    if (item.config && item.config.assignedColumns){
                        if (item.config.assignedColumns.numDataColumn) {
                            dataMapping.columns.push(createDataMappingColumn(item.config.assignedColumns.numDataColumn));
                        }
                        if (item.config.assignedColumns.textDataColumn) {
                            dataMapping.columns.push(createDataMappingColumn(item.config.assignedColumns.textDataColumn));
                        }
                        if (item.config.assignedColumns.dateDataColumn) {
                            dataMapping.columns.push(createDataMappingColumn(item.config.assignedColumns.dateDataColumn));
                        }
                        var furtherColumns = item.config.assignedColumns.furtherColumns;
                        if (furtherColumns) {
                            angular.forEach(furtherColumns, function(anyColumn) {
                                dataMapping.columns.push(createDataMappingColumn(anyColumn));
                            });
                        }
                    }
                    return dataMapping;
                }
            };
        }
    ]);